from typing import Callable, Generator, TypeVar
from operator import attrgetter as get_attr

SI = TypeVar('SI', str, int)


def filter_attr(generator: Generator, attr: str, op: Callable[[SI], bool], value: SI) -> Generator:
    """Returns a generator based on filter"""
    return (file for file in generator if op(get_attr(attr)(file if attr == "name" else file.stat()), value))



from typing import Iterator


def print_iterator(it: Iterator) -> None:
    """Prints the elements of the provided iterator"""
    try:
        while True:
            file = next(it)
            print(file.name)
    except StopIteration:
        pass



from os import scandir
from typing import Generator


def generator_from_path(path: str, max_depth: int = 1, cur_depth: int = 1) -> Generator:
    """Retrieves files and directories from path recursively, up to max depth"""
    for entry in scandir(path):
        if entry.is_dir() and cur_depth < max_depth:
            yield from generator_from_path(entry.path, max_depth=max_depth, cur_depth=cur_depth+1)
        elif entry.is_file():
            yield entry



import click
import operator
import time
from os import getcwd
from zoek.generator_from_path import generator_from_path
from zoek._filter_attr import filter_attr
from zoek.print_iterable import print_iterator
from zoek._custom_operators import string_contains, string_begins_with


@click.command()
@click.argument('path', default=getcwd())
@click.option('--depth', '-d', type=int, default=1, help="how many subdirectories should zoek look in?"
                                                         "defaults to 1, meaning only files in current directory.")
@click.option('--startswith', '-s', type=str, help="require file name to start with a string")
@click.option('--contains', '-c', type=str, help="require file name to contain a certain string")
@click.option('--minsize', '-m', type=int,
              help="filter on size (-size) given in Kb: the file has size n, where n could be:"
                   "+integer, integer -> file size bigger than integer"
                   "-integer -> file size smaller than integer")
@click.option('--datecreated', '-dc', type=int, help="files that were created n minutes ago, where n could be:"
                                                     "+integer -> files created more than n minutes ago"
                                                     "-integer -> files created less than n minutes ago"
                                                     "integer -> files created exactly n minutes ago)")
@click.option('--datemodified', '-dm', type=int, help="files that were modified n minutes ago, where n could be:"
                                                      "+integer -> files created more than n minutes age"
                                                      "-integer -> files created less than n minutes age"
                                                      "integer -> files created exactly n minutes ago")
def main(depth, path, startswith, minsize, contains, datecreated, datemodified):
    files = generator_from_path(path, max_depth=depth)
    time_now = time.time()

    if startswith is not None:
        files = filter_attr(generator=files, attr="name", op=string_begins_with, value=startswith)

    if contains is not None:
        files = filter_attr(generator=files, attr="name", op=string_contains, value=contains)

    if minsize is not None:
        files = filter_attr(generator=files, attr="st_size", op=operator.ge, value=minsize)

    if datecreated is not None:
        diff = time_now - (abs(datecreated) * 60)
        if datecreated <= 0:
            ops = operator.ge
        else:
            ops = operator.lt

        files = filter_attr(generator=files, attr="st_ctime", op=ops, value=diff)

    if datemodified is not None:
        diff = time_now - (abs(datemodified) * 60)
        if datemodified <= 0:
            ops = operator.ge
        else:
            ops = operator.lt
        files = filter_attr(generator=files, attr="st_mtime", op=ops, value=diff)

    print_iterator(files)


if __name__ == '__main__':
    main()



def string_contains(string: str, substr: str) -> bool:
    """Determines whether substring is part of string or not
    >>> string_contains("hello", "he")
    True
    >>> string_contains("hello", "wo")
    False
    """
    return substr in string


def string_begins_with(string: str, substr: str) -> bool:
    """Determines whether string starts with substring or not
    >>> string_begins_with("hello", "he")
    True
    >>> string_begins_with("hello", "wo")
    False
    """
    return string.startswith(substr)



from setuptools import setup, find_packages

base_requirements = [
    'Click>=7.0.0',
]

dev_requirements = [
    'pytest',
    'pytest-flake8',
    'pytest-cov',
    'setuptools>=38.6.0',
    'twine>=1.11.0',
    'wheel>=0.31.0',
]

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="zoek",
    version="0.0.2",
    author="Davey Kreeft, Tein de Vries",
    author_email="dkreeft@xccelerated.io, teindevries@gmail.com",
    description="Command-line utility to search for files and directories",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/dkreeft/zoek",
    packages=find_packages('src'),
    package_dir={'': 'src'},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=base_requirements,
    extras_require={
        'dev': dev_requirements
    },
    entry_points={
        'console_scripts': [
            'zoek = zoek.cli:main'
        ]
    }
)
